#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <cutil.h>

#include "../Header/CPUGausSeidel.h"
#include "../Header/GPUGausSeidel.h"
#include "../Header/matrixInversion.h"


#define THRESHHOLD 0.00001

void main ()
{
    int i, j;
    int memresult = 0;
    int size = 256;
    float *dataInput = (float*) malloc (sizeof (float) * size * size);
    float *resultGPU = (float*) malloc (sizeof (float) * size * size);
    float *resultCPU = (float*) malloc (sizeof (float) * size * size);
    float* tmp;
    float* tmp2;
    unsigned int timer = 0;

    srand ( time(NULL) );

    //generate some test data
    tmp = dataInput;
    for (i = 0; i < size; i++)
    {
       for (j = 0; j < size; j++)
       {
           tmp[j] = 255 * (float) rand ();
       }
       tmp += size;
    }
    printf ("Matrix size: %dx%d\n", size, size);
    //warmup
    matrixInv (dataInput, resultGPU, size, 1);
    cutCreateTimer(&timer);
    cutStartTimer(timer);
    for (i = 0; i < 10; i++)
    {
        matrixInv (dataInput, resultGPU, size, 1);
    }
    cutStopTimer(timer);    
    printf ("GPU Time %lfms\n", cutGetTimerValue(timer) / 10);
    cutDeleteTimer(timer);
 
    timer = 0;
    
    cutCreateTimer(&timer);
    cutStartTimer(timer);    

    for (i = 0; i < 10; i++)
    {
        matrixInv (dataInput, resultCPU, size, 0);
    }

    cutStopTimer(timer);
    printf ("CPU Time %lfms\n", cutGetTimerValue (timer) / 10);
    cutDeleteTimer(timer);

    printf ("validating gpu Result..\n");

    tmp = resultCPU;
    tmp2 = resultGPU;

    for (i = 0; i < size; i++)
    {
        for (j = 0; j < size; j++)
        {
            if (fabs (tmp[j] - tmp2[j]) > THRESHHOLD)
                memresult++;
        }
        tmp += size;
        tmp2 += size;
    }
    
    if (memresult == 0)
    {
        printf ("result matched\n");
    }
    else
    {
        printf ("result mismached %d times with norm %f\n", memresult, THRESHHOLD);
    }  
    free (resultCPU);
    free (resultGPU);
    free (dataInput);
    getchar ();
}
