/********************************************************************
*  File: matrixInv.c
*
*  Description:
*	
*	Wrapperfunction for the CPU/GPU Matrix inversion.
*
*  Arguments: 
*	- float *hDataIn      Input Matrix 1D, no data changes
*   - float *hDataOut     Output Matrix 1D, the inverse datamatrix  
*   - int size            Matrix dimension in size, width = height = size
*   - int useGPU          Set this Flag to 0 to use a native CPU implementation
*                         else it will run the CUDA runtime.  
*                           
*	
* Used custom Routines:
*	- CPUGausSeidel ()
*   - GPUGausSeidel ()
*
*********************************************************************/

#include "../Header/matrixInversion.h"
#include "../Header/CPUGausSeidel.h"
#include "../Header/GPUGausSeidel.h"


DOEXPORT int matrixInv (float *dataIn, float *dataOut, int size, int useGPU)
{
    
    if (useGPU == 0)
    {
        //run the CPU implementation 
        return CPUGausSeidel (dataIn, dataOut, size);
    }
    //run the GPU implementation
    return GPUGausSeidel (dataIn, dataOut, size);
}
