%Testumgebung fr die Matrixinversion in Matlab, Ansi-c, CUDA
%
%   Die Testumgebung generiert eine quadratische Zufallsmatrix im Intervall
%   von 0-255. Durch die Multiplikation von "Input = Input * Input'" wird aus der
%   Eingangsmatrix eine positiv definierte Matrix.
%   Fr die Matrixinversion werden die Zeiten gemessen der nativen-c, 
%   CUDA und Matlab implementierungen.
%   Der Ergebis Plot zeigt ein Residual zwischen der Matlab double/float
%   zu CUDA implementierung sowie das Residual von Matlab zu nativ-c.
%
%   Angewendetes Modul: MatrixInv
%   
%   Abhngige Daten: MatrixInversion.dll
%                    MatrixInversion.lib
%                    MatrixInversion.h
%               
%
%   20.10.2008 Ch. Wagner @ HS Mannheim


clear();
close all;

SIZE = 64;  %Inputmatrix Dimension 
USEMATLABDOUBLE = 1;

%MatrixInv Moduel in Matlab einbinden, falls bereits geladen vorher
%entladen
if libisloaded('MatrixInversion') == 1
    unloadlibrary('MatrixInversion');
end
loadlibrary('MatrixInversion', 'matrixInversion.h')

%Zufallsdaten generieren und positiv-definierte Matrix erzeugen
if (USEMATLABDOUBLE == 0)
    Input = single ((255).*rand (SIZE));
else   
    Input = (255).*rand (SIZE); 
end
Input = Input * Input';

%Erzeugen der Ergebnismatritzen, geht evtl schner
for i = 1:SIZE
    for j = 1:SIZE 
        cpuResult(i,j) = 1;
        gpuResult(i,j) = 1;
        matlabResult (i,j) = 1;
    end
end

%Erzeuge Datenobjekte fr das C-Modul (Copy By Value => keine probleme bei Konvertierung von double2float)
inputPtr = libpointer('singlePtr', Input);
gpuResultPtr = libpointer('singlePtr', gpuResult);
cpuResultPtr = libpointer('singlePtr', cpuResult);


%Warmup
calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
%fteres Messen fr die Zeitenberechnung der GPU Funktion
disp('GPU Time in sec: ');
tic
    %Unrolled 10 times for time messurement
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
    calllib('MatrixInversion', 'matrixInv', inputPtr, gpuResultPtr, SIZE, 1);
timeGPU = toc();

disp(mean(timeGPU)/10);

%fteres Messen fr die Zeitenberechnung der CPU Funktion
disp('CPU Time in sec: ');
tic
    %Unrolled 10 times for time messurement
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
    calllib('MatrixInversion', 'matrixInv', inputPtr, cpuResultPtr, SIZE, 0);
timeCPU = toc();
disp(mean(timeCPU)/10);

%Einmaliges Messen der Zeitdauer von Matlab
disp('Matlab Time in sec: ');
tic
    matlabResult = inv (Input);
timeMatlab = toc();

disp (timeMatlab);

%Ergebnis Objekte aus dem Modul zurck in Matlab kopieren
gpuResult = get (gpuResultPtr, 'Value');
cpuResult = get (cpuResultPtr, 'Value');

%Berechne das Resiudal zwischen der Matlab und c Implementierung
%sowie das Residual zwischen Matlab und der CUDA Implementierung
residualCPUvsMatlab = abs (matlabResult - cpuResult);
residualGPUvsMatlab = abs (matlabResult - gpuResult);

%Ausgabe der Residualblocks. Zu sehen ist, dass die Rechengenauigkeit von
%"Matlab double zu c float" eine Fehlertoleranz von ca xx% besitzt. Wird
%Matlab auf float gestellt fllt die Fehlertoleranz auf ca xx%.
%
h = figure;
subplot (2,1,1), imagesc (residualGPUvsMatlab); figure(h), title('Residual GPU float vs Matlab double');
subplot (2,1,2), imagesc (residualCPUvsMatlab); figure(h), title('Residual CPU float vs Matlab double');

%Modul entladen
unloadlibrary('MatrixInversion');


